/**
 * \file hal.h
 *
 * \brief PIC24F Hardware abstraction header file
 *
 * Copyright (c) 2016 Measurement Specialties. All rights reserved.
 *
 */

#ifndef __HAL_H__
#define	__HAL_H__

#include <p24FJ128GA010.h>
#include <stdbool.h>
#include <stdint.h>
#include <stddef.h>

/*
 * system clocks
 *
 */
#define XTAL    8000000L             // Crystal of Explorer 8MHz
#define FRC_OSC 8000000L             // internal fast RC 8MHz
#define CLK_DIV_PERIPH  1           // value at reset
#define CLK_DIV_CPU     1           // value at reset
#define FOSC    FRC_OSC/CLK_DIV_CPU
#define FCPU    FOSC/2              // CPU clock 4MHz about 4MIPS
#define FCY     FCPU

/*
 * timer1
 *
 */
#define T1CS_FOSC   0           // internal clock is sourced FOSC/2
#define T1CKPS_256  0b11        // Prescaler 256  Frequence Timer1 = 15,625KHz
#define T1cy        15625
#define T1ON        1
#define T1OFF       0

#define T1_1sec     (1*T1cy)       // T1cy is at 64s
#define T1_500ms    0.5*T1cy
#define T1_250ms    0.25*T1cy

/*
 * delay functions
 *
 */
extern void __delay32(unsigned long cycles);

#define delay_ms(d) { __delay32( (unsigned long) (((unsigned long long) d)*(FCY)/1000ULL)); }
#define delay_us(d) { __delay32( (unsigned long) (((unsigned long long) d)*(FCY)/1000000ULL)); }

/*
 * GPIOs
 *
 */
#define TRIS_SW3			_TRISD6
#define SW3					_RD6

#define TRIS_SW4			_TRISD13
#define SW4					_RD13

#define TRIS_SW6			_TRISD7
#define SW6					_RD7

#define INPUT				1
#define OUTPUT				0
#define HIGH				1
#define LOW					0

#define TRIS_HTU21D			_TRISE8
#define PORT_HTU21D			_LATE8

#define TRIS_MS5637			_TRISB0
#define PORT_MS5637			_LATB0

#define TRIS_TSYS01			_TRISB1
#define PORT_TSYS01			_LATB1

#define TRIS_MS8607			_TRISB3
#define PORT_MS8607			_LATB3

#define disable_htu21d()	TRIS_HTU21D = INPUT
#define enable_htu21d()		disable_all(); TRIS_HTU21D = OUTPUT; PORT_HTU21D = LOW; delay_ms(50)

#define disable_ms5637()	TRIS_MS5637 = INPUT
#define enable_ms5637()		disable_all(); TRIS_MS5637 = OUTPUT; PORT_MS5637 = LOW; delay_ms(50)

#define disable_tsys01()	TRIS_TSYS01 = INPUT
#define enable_tsys01()		disable_all(); TRIS_TSYS01 = OUTPUT; PORT_TSYS01 = LOW; delay_ms(50)

#define disable_ms8607()	TRIS_MS8607 = INPUT
#define enable_ms8607()		disable_all(); TRIS_MS8607 = OUTPUT; PORT_MS8607 = LOW; delay_ms(50)

#define disable_all()       disable_htu21d(); disable_ms5637(); disable_tsys01(); disable_ms8607()

#define is_sw3_pressed()    (SW3 == LOW)
#define is_sw4_pressed()    (SW4 == LOW)

#endif